@@ -95,13 +95,14 @@ where
 }
 
 #[cfg(feature = "std")]
-impl<'b, T> Decode<'b> for std::collections::HashSet<T>
+impl<'b, T, S> Decode<'b> for std::collections::HashSet<T, S>
 where
-    T: Decode<'b> + Eq + std::hash::Hash
+    T: Decode<'b> + Eq + std::hash::Hash,
+    S: std::hash::BuildHasher + std::default::Default
 {
     fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
         let iter: ArrayIter<T> = d.array_iter()?;
-        let mut v = std::collections::HashSet::new();
+        let mut v = std::collections::HashSet::default();
         for x in iter {
             v.insert(x?);
         }
@@ -125,13 +126,14 @@ where
 }
 
 #[cfg(feature = "std")]
-impl<'b, K, V> Decode<'b> for std::collections::HashMap<K, V>
+impl<'b, K, V, S> Decode<'b> for std::collections::HashMap<K, V, S>
 where
     K: Decode<'b> + Eq + std::hash::Hash,
-    V: Decode<'b>
+    V: Decode<'b>,
+    S: std::hash::BuildHasher + std::default::Default
 {
     fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
-        let mut m = std::collections::HashMap::new();
+        let mut m = std::collections::HashMap::default();
         let iter: MapIter<K, V> = d.map_iter()?;
         for x in iter {
             let (k, v) = x?;
@@ -176,6 +178,12 @@ impl<'b> Decode<'b> for () {
     }
 }
 
+impl<'b, T: Decode<'b>> Decode<'b> for core::num::Wrapping<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.decode().map(core::num::Wrapping)
+    }
+}
+
 #[cfg(target_pointer_width = "32")]
 impl<'b> Decode<'b> for usize {
     fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
@@ -241,6 +249,32 @@ decode_nonzero! {
     core::num::NonZeroI64, "unexpected 0 when decoding a `NonZeroI64`"
 }
 
+macro_rules! decode_atomic {
+    ($($t:ty)*) => {
+        $(
+            impl<'b> Decode<'b> for $t {
+                fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+                    d.decode().map(<$t>::new)
+                }
+            }
+        )*
+    }
+}
+
+decode_atomic! {
+    core::sync::atomic::AtomicBool
+    core::sync::atomic::AtomicU8
+    core::sync::atomic::AtomicU16
+    core::sync::atomic::AtomicU32
+    core::sync::atomic::AtomicU64
+    core::sync::atomic::AtomicUsize
+    core::sync::atomic::AtomicI8
+    core::sync::atomic::AtomicI16
+    core::sync::atomic::AtomicI32
+    core::sync::atomic::AtomicI64
+    core::sync::atomic::AtomicIsize
+}
+
 #[cfg(feature = "alloc")]
 macro_rules! decode_sequential {
     ($($t:ty, $push:ident)*) => {
@@ -372,6 +406,48 @@ impl<'b> Decode<'b> for core::time::Duration {
     }
 }
 
+#[cfg(feature = "std")]
+impl<'b> Decode<'b> for std::time::SystemTime {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        std::time::UNIX_EPOCH
+            .checked_add(d.decode()?)
+            .ok_or(Error::Message("duration value can not represent system time"))
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::cell::Cell<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.decode().map(core::cell::Cell::new)
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::cell::RefCell<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.decode().map(core::cell::RefCell::new)
+    }
+}
+
+#[cfg(feature = "std")]
+impl<'a, 'b: 'a> Decode<'b> for &'a std::path::Path {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.str().map(std::path::Path::new)
+    }
+}
+
+#[cfg(feature = "std")]
+impl<'b> Decode<'b> for Box<std::path::Path> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.decode().map(std::path::PathBuf::into_boxed_path)
+    }
+}
+
+#[cfg(feature = "std")]
+impl<'b> Decode<'b> for std::path::PathBuf {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        d.decode().map(std::path::Path::to_path_buf)
+    }
+}
+
 #[cfg(feature = "std")]
 impl<'b> Decode<'b> for std::net::IpAddr {
     fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
@@ -438,3 +514,63 @@ impl<'b> Decode<'b> for std::net::SocketAddrV6 {
     }
 }
 
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::Range<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        decode_fields! { d |
+            0 start => T ; "Range::start"
+            1 end   => T ; "Range::end"
+        }
+        Ok(core::ops::Range { start, end })
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::RangeFrom<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        decode_fields! { d |
+            0 start => T ; "RangeFrom::start"
+        }
+        Ok(core::ops::RangeFrom { start })
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::RangeTo<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        decode_fields! { d |
+            0 end => T ; "RangeTo::end"
+        }
+        Ok(core::ops::RangeTo { end })
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::RangeToInclusive<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        decode_fields! { d |
+            0 end => T ; "RangeToInclusive::end"
+        }
+        Ok(core::ops::RangeToInclusive { end })
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::RangeInclusive<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        decode_fields! { d |
+            0 start => T ; "RangeInclusive::start"
+            1 end   => T ; "RangeInclusive::end"
+        }
+        Ok(core::ops::RangeInclusive::new(start, end))
+    }
+}
+
+impl<'b, T: Decode<'b>> Decode<'b> for core::ops::Bound<T> {
+    fn decode(d: &mut Decoder<'b>) -> Result<Self, Error> {
+        if Some(2) != d.array()? {
+            return Err(Error::Message("expected enum (2-element array)"))
+        }
+        match d.u32()? {
+            0 => d.decode().map(core::ops::Bound::Included),
+            1 => d.decode().map(core::ops::Bound::Excluded),
+            2 => d.limited_skip().map(|_| core::ops::Bound::Unbounded),
+            n => Err(Error::UnknownVariant(n))
+        }
+    }
+}
